package
{

	import away3d.containers.View3D;
	import away3d.debug.AwayStats;
	import away3d.entities.Mesh;
	import away3d.lights.PointLight;
	import away3d.materials.TextureMaterial;
	import away3d.materials.lightpickers.StaticLightPicker;
	import away3d.primitives.Plane;
	import away3d.textures.BitmapTexture;

	import flash.display.BitmapData;

	import flash.display.Sprite;
	import flash.events.Event;

	[SWF(width="1024", height="576", frameRate="60", backgroundColor="0x000000")]
	public class ReuseDemo extends Sprite
	{
		private static const SHARE_GEOMETRY : Boolean = true;
		private static const SHARE_MATERIAL : Boolean = true;

		private var _view : View3D;

		[Embed(source="/../assets/rockbase.jpg")]
		private var Albedo : Class;

		[Embed(source="/../assets/rockbase_local.png")]
		private var Normals : Class;

		private var _referenceMesh : Mesh;
		private var _referenceMaterial : TextureMaterial;
		private var _light : PointLight;

		private var _albedo : BitmapData = new Albedo().bitmapData;
		private var _normals : BitmapData = new Normals().bitmapData;

		private var _maxMeshes : uint = 2500;
		private var _numMeshes : uint = 0;

		public function ReuseDemo()
		{
			initView();

			addEventListener(Event.ENTER_FRAME, onEnterFrame);
		}

		private function initView() : void
		{
			_view = new View3D();
			_view.camera.lens.far = 7000;
			_view.camera.x = 0;
			_view.camera.y = 0;
			_view.camera.z = 0;
			_light = new PointLight();
			_view.scene.addChild(_light);
			addChild(_view);
			addChild(new AwayStats(_view));
		}

		private function createMaterial() : TextureMaterial
		{
			var texture : BitmapTexture = new BitmapTexture(_albedo);
			var material : TextureMaterial = new TextureMaterial(texture);
			material.normalMap = new BitmapTexture(_normals);
			material.bothSides = true;
			material.lightPicker = new StaticLightPicker([ _light ]);
			return material;
		}

		private function createMesh() : Mesh
		{
			// share Geometry & Material
			if (SHARE_GEOMETRY && SHARE_MATERIAL) {
				// create if not exists
				if (!_referenceMesh)
					_referenceMesh = new Plane(createMaterial());

				// simply clone reference mesh
				return Mesh(_referenceMesh.clone());
			}

			// Share Geometry only
			if (SHARE_GEOMETRY) {
				// create reference mesh if not exists
				if (!_referenceMesh) _referenceMesh = new Plane();

				// use geometry from reference mesh, but do not reuse material
				return new Mesh(createMaterial(), _referenceMesh.geometry);
			}

			// Share Material only
			if (SHARE_MATERIAL) {
				if (!_referenceMaterial)
					_referenceMaterial = createMaterial();
				// create new plane (will create new Geometry), and share material
				return new Plane(_referenceMaterial);
			}

			// Share nothing!
			return new Plane(createMaterial());
		}

		private function onEnterFrame(event : Event) : void
		{
			_view.camera.rotationY++;

			if (_numMeshes < _maxMeshes)
				createMeshes();

			_view.render();
		}

		private function createMeshes() : void
		{
			var mesh : Mesh;
			var i : uint;

			while (i++ < 10 && _numMeshes < _maxMeshes) {
				++_numMeshes;
				mesh = createMesh();
				mesh.x = Math.random()*10000 - 5000;
				mesh.y = Math.random()*10000 - 5000;
				mesh.z = Math.random()*10000 - 5000;
				mesh.rotationX = Math.random()*360;
				mesh.rotationZ = Math.random()*360;
				_view.scene.addChild(mesh);
			}
		}
	}
}
